@@ -1,15 +1,27 @@
 import sys
+import os
 import pathlib
 
-__version__ = '1.0.2'
+__version__ = '1.8.2'
+
+# The variable _in_notebook is used to enable audio playing in Jupiter notebooks
+# and on Google colab (see slab.sound.play())
+try:
+    shell = get_ipython().__class__.__module__
+    if 'terminal' in shell:
+        _in_notebook = False
+    else:
+        _in_notebook = True  # probably in a notebook
+except NameError:
+    _in_notebook = False  # probably standard Python interpreter
 
 sys.path.append('..\\')
 
-from slab.hrtf import HRTF
+from slab.hrtf import HRTF, Room
 from slab.psychoacoustics import *
 from slab.binaural import Binaural
-from slab.sound import Sound, set_default_level, set_calibration_intensity, calibrate
-from slab.signal import Signal, set_default_samplerate
+from slab.sound import Sound, set_default_level, set_calibration_intensity, get_calibration_intensity, calibrate
+from slab.signal import Signal, set_default_samplerate, get_default_samplerate
 from slab.filter import Filter
 
 def cite(fmt='bibtex'):
