@@ -1,5 +1,9 @@
 import copy
 import numpy
+import math
+import sys
+import pathlib
+import pickle
 from slab.sound import Sound
 from slab.signal import Signal
 from slab.filter import Filter
@@ -16,6 +20,9 @@ class Binaural(Sound):
         data (slab.Signal | numpy.ndarray | list | str): see documentation of slab.Sound for details. the `data` must
             have either one or two channels. If it has one, that channel is duplicated
         samplerate (int): samplerate in Hz, must only be specified when creating an instance from an array.
+        name (str): A string label for the Sound object. The inbuilt sound generating functions will automatically
+            set .name to the name of the method used. Useful for logging during experiments.
+
     Attributes:
         .left: the first data channel, containing the sound for the left ear.
         .right: the second data channel, containing the sound for the right ear
@@ -23,7 +30,8 @@ class Binaural(Sound):
         .n_channels: the number of channels in `data`. Must be 2 for a binaural sound.
         .n_samples: the number of samples in `data`. Equals `duration` * `samplerate`.
         .duration: the duration of the sound in seconds. Equals `n_samples` / `samplerate`.
-    """
+        .name: string label of the sound.
+   """
     # instance properties
     def _set_left(self, other):
         if hasattr(other, 'samplerate'):  # probably an slab object
@@ -42,8 +50,12 @@ class Binaural(Sound):
     right = property(fget=lambda self: Sound(self.channel(1)), fset=_set_right,
                      doc='The right channel for a stereo sound.')
 
-    def __init__(self, data, samplerate=None):
+    def __init__(self, data, samplerate=None, name='unnamed'):
         if isinstance(data, (Sound, Signal)):
+            if hasattr(data, 'name'):
+                self.name = data.name
+            else:
+                self.name = name
             if data.n_channels == 1:  # if there is only one channel, duplicate it.
                 self.data = numpy.tile(data.data, 2)
             elif data.n_channels == 2:
@@ -51,24 +63,24 @@ class Binaural(Sound):
             else:
                 raise ValueError("Data must have one or two channel!")
             self.samplerate = data.samplerate
-        elif isinstance(data, (list, tuple)):
+        elif isinstance(data, (list, tuple)): # list of Sounds
             if isinstance(data[0], (Sound, Signal)):
                 if data[0].n_samples != data[1].n_samples:
                     raise ValueError('Sounds must have same number of samples!')
                 if data[0].samplerate != data[1].samplerate:
                     raise ValueError('Sounds must have same samplerate!')
-                super().__init__([data[0].data[:, 0], data[1].data[:, 0]], data[0].samplerate)
-            else:
-                super().__init__(data, samplerate)
-        elif isinstance(data, str):
+                super().__init__([data[0].data[:, 0], data[1].data[:, 0]], data[0].samplerate, name=data[0].name)
+            else: # list of samples
+                super().__init__(data, samplerate, name=name)
+        elif isinstance(data, str): # file name
             super().__init__(data, samplerate)
             if self.n_channels == 1:
                 self.data = numpy.tile(self.data, 2)  # duplicate channel if monaural file
-        else:
-            super().__init__(data, samplerate)
+        else: # anything but Sound, list, or file name
+            super().__init__(data, samplerate, name=name)
             if self.n_channels == 1:
                 self.data = numpy.tile(self.data, 2)  # duplicate channel if monaural file
-        if self.n_channels != 2:
+        if self.n_channels != 2: # bail if unable to enforce 2 channels
             ValueError('Binaural sounds must have two channels!')
 
     def itd(self, duration=None, max_lag=0.001):
@@ -94,7 +106,9 @@ class Binaural(Sound):
         """
         if duration is None:
             return self._get_itd(max_lag)
-        return self._apply_itd(duration)
+        out = copy.deepcopy(self)
+        out.name = f'{str(duration)}-itd_{self.name}'
+        return out._apply_itd(duration)
 
     def _get_itd(self, max_lag):
         max_lag = Sound.in_samples(max_lag, self.samplerate)
@@ -117,12 +131,15 @@ class Binaural(Sound):
         """
         Either estimate the interaural level difference of the sound or generate a new sound with the specified
         interaural level difference. Negative ILD value means that the left channel is louder than the right
-        channel, meaning that the sound source is to the left. The level difference is achieved by adding half the ILD
-        to one channel and subtracting half from the other channel, so that the mean intensity remains constant.
+        channel, meaning that the sound source is to the left.
+        If a single value is provided, the level difference is achieved by adding half the ILD to one channel
+        and subtracting half from the other channel, so that the mean intensity remains constant.
+        If a tuple is provided, levels are added separately to each channel.
 
         Arguments:
-            dB (None | int | float): If None, estimate the sound's ITD. Given a value, a new sound is generated with
-                the desired interaural level difference in decibels.
+            dB (None | int | float | tuple): If None, estimate the sound's ILD. Given a single value, a new sound
+                is generated with the desired interaural level difference in decibels. Given a tuple, a sound is
+                generated with the desired binaural levels in decibels.
         Returns:
             (float | slab.Binaural): The sound's interaural level difference, or a new sound with the specified ILD.
         Examples::
@@ -133,11 +150,17 @@ class Binaural(Sound):
         """
         if dB is None:
             return self.right.level - self.left.level
-        new = copy.deepcopy(self)  # so that we can return a new sound
+        out = copy.deepcopy(self)  # so that we can return a new sound
         level = numpy.mean(self.level)
-        new_levels = (level - dB/2, level + dB/2)
-        new.level = new_levels
-        return new
+        if type(dB) == int or type(dB) == float:
+            out_levels = (level - dB / 2, level + dB / 2)
+        elif type(dB) == tuple:
+            out_levels = (level + dB[0], level + dB[1]) #todo ensure that overall loudness doesnt change
+        else:
+            raise TypeError("Argument 'dB' must be an integer, float or a tuple.")
+        out.level = out_levels
+        out.name = f'{str(dB)}-ild_{self.name}'
+        return out
 
     def itd_ramp(self, from_itd=-6e-4, to_itd=6e-4):
         """
@@ -156,7 +179,7 @@ class Binaural(Sound):
             moving = sig.itd_ramp(from_itd=-0.001, to_itd=0.01)
             moving.play()
         """
-        new = copy.deepcopy(self)
+        out = copy.deepcopy(self)
         # make the ITD ramps
         left_ramp = numpy.linspace(from_itd / 2, to_itd / 2, self.n_samples)
         right_ramp = numpy.linspace(-from_itd / 2, -to_itd / 2, self.n_samples)
@@ -166,9 +189,10 @@ class Binaural(Sound):
             filter_length = self.n_samples // 16 * 2  # 1/8th of n_samples, always even
         else:
             raise ValueError('Signal too short! (min 512 samples)')
-        new = new.delay(duration=left_ramp, channel=0, filter_length=filter_length)
-        new = new.delay(duration=right_ramp, channel=1, filter_length=filter_length)
-        return new
+        out = out.delay(duration=left_ramp, channel=0, filter_length=filter_length)
+        out = out.delay(duration=right_ramp, channel=1, filter_length=filter_length)
+        out.name = f'ild-ramp_{self.name}'
+        return out
 
     def ild_ramp(self, from_ild=-50, to_ild=50):
         """
@@ -188,16 +212,17 @@ class Binaural(Sound):
             moving = sig.ild_ramp(from_ild=-10, to_ild=10)
             move.play()
         """
-        new = self.ild(0)  # set ild to zero
+        out = self.ild(0)  # set ild to zero
         # make ramps
         left_ramp = numpy.linspace(-from_ild / 2, -to_ild / 2, self.n_samples)
         right_ramp = numpy.linspace(from_ild / 2, to_ild / 2, self.n_samples)
         left_ramp = 10**(left_ramp/20.)
         right_ramp = 10**(right_ramp/20.)
         # multiply channels with ramps
-        new.data[:, 0] *= left_ramp
-        new.data[:, 1] *= right_ramp
-        return new
+        out.data[:, 0] *= left_ramp
+        out.data[:, 1] *= right_ramp
+        out.name = f'ild-ramp_{self.name}'
+        return out
 
     @staticmethod
     def azimuth_to_itd(azimuth, frequency=2000, head_radius=8.75):
@@ -209,7 +234,7 @@ class Binaural(Sound):
         Arguments:
             azimuth (int | float): The azimuth angle of the sound source, negative numbers refer to sources to the left.
             frequency (int | float): Frequency in Hz for which the ITD is estimated.
-                Use the default for for sounds with a broadband spectrum.
+                Use the default for sounds with a broadband spectrum.
             head_radius (int | float): Radius of the head in centimeters. The bigger the head, the larger the ITD.
         Returns:
             (float): The interaural time difference for a sound source at a given azimuth.
@@ -218,14 +243,22 @@ class Binaural(Sound):
             # compute the ITD for a sound source 90 degrees to the left for a large head
             itd = slab.Binaural.azimuth_to_itd(-90, head_radius=10)
         """
-        head_radius = head_radius / 100
-        azimuth_radians = numpy.radians(azimuth)
         speed_of_sound = 344  # m/s
-        itd_2000 = (head_radius / speed_of_sound) * \
-            (azimuth_radians + numpy.sin(azimuth_radians))  # Woodworth
-        itd_500 = (3 * head_radius / speed_of_sound) * numpy.sin(azimuth_radians)
+        head_radius = head_radius / 100  # convert cm to meters
+        azimuth = azimuth % 360  # wrap to [0, 360]
+        if azimuth > 180:
+            return -Binaural.azimuth_to_itd(360 - azimuth, frequency, head_radius * 100)
+        if azimuth < 0:
+            return -Binaural.azimuth_to_itd(-azimuth, frequency, head_radius * 100)
+        azimuth_rad = numpy.radians(azimuth)
+        if azimuth_rad <= numpy.pi / 2:  # 0° to 90°
+            itd_2000 = (head_radius / speed_of_sound) * (azimuth_rad + numpy.sin(azimuth_rad))
+            itd_500 = (3 * head_radius / speed_of_sound) * numpy.sin(azimuth_rad)
+        else:  # 90° to 180°
+            itd_2000 = (head_radius / speed_of_sound) * (numpy.pi - azimuth_rad + numpy.sin(azimuth_rad))
+            itd_500 = (3 * head_radius / speed_of_sound) * (numpy.sin(azimuth_rad))
         itd = numpy.interp(frequency, [500, 2000], [itd_500, itd_2000],
-                           left=itd_500, right=itd_2000)
+                           left=itd_500, right=itd_2000)  # interpolate based on frequency
         return itd
 
     @staticmethod
@@ -235,13 +268,13 @@ class Binaural(Sound):
 
         Arguments:
             azimuth (int | float): The azimuth angle of the sound source, negative numbers refer to sources to the left.
-            frequency (int | float): Frequency in Hz for which the ITD is estimated.
-                Use the default for for sounds with a broadband spectrum.
+            frequency (int | float): Frequency in Hz for which the ILD is estimated.
+                Use the default for sounds with a broadband spectrum.
             ils (dict | None): interaural level spectrum from which the ILD is taken. If None,
             `make_interaural_level_spectrum()` is called. For repeated use, it is better to generate and keep the ils in
             a variable to avoid re-computing it.
         Returns:
-            (float): The interaural level difference for a sound source at a given azimuth in decibels.
+            (tuple): The binaural levels for a sound source at a given azimuth in decibels.
         Examples::
 
             ils = slab.Binaural.make_interaural_level_spectrum() # using default KEMAR HRTF
@@ -249,10 +282,15 @@ class Binaural(Sound):
         """
         if ils is None:
             ils = Binaural.make_interaural_level_spectrum()
+        level_diffs_left = ils['level_diffs'][0]
+        level_diffs_right = ils['level_diffs'][1]
         # interpolate levels at azimuth:
-        level_diffs = ils['level_diffs']
-        levels = [numpy.interp(azimuth, ils['azimuths'], level_diffs[i, :]) for i in range(level_diffs.shape[0])]
-        return numpy.interp(frequency, ils['frequencies'], levels)*-1   # interpolate level difference at frequency
+        levels_right = [numpy.interp(azimuth, ils['azimuths'], level_diffs_right[i, :]) for i in
+                        range(level_diffs_right.shape[0])]
+        levels_left = [numpy.interp(azimuth, ils['azimuths'], level_diffs_left[i, :]) for i in
+                       range(level_diffs_left.shape[0])]
+        return (numpy.interp(frequency, ils['frequencies'], levels_left),
+                            numpy.interp(frequency, ils['frequencies'], levels_right))
 
     def at_azimuth(self, azimuth=0, ils=None):
         """
@@ -273,6 +311,7 @@ class Binaural(Sound):
         itd = Binaural.azimuth_to_itd(azimuth, frequency=centroid)
         ild = Binaural.azimuth_to_ild(azimuth, frequency=centroid, ils=ils)
         out = self.itd(duration=itd)
+        out.name = f'{azimuth}-azi_{self.name}'
         return out.ild(dB=ild)
 
     def externalize(self, hrtf=None):
@@ -289,7 +328,8 @@ class Binaural(Sound):
         if hrtf is None:
             hrtf = HRTF.kemar()  # load KEMAR as default
         # get HRTF for [0,0] direction:
-        idx_frontal = numpy.where((hrtf.sources[:, 1] == 0) & (hrtf.sources[:, 0] == 0))[0][0]
+        idx_frontal = numpy.where((hrtf.sources.vertical_polar[:, 1] == 0) &
+                                  (hrtf.sources.vertical_polar[:, 0] == 0))[0][0]
         if not idx_frontal.size: # idx_frontal is empty
             raise ValueError('No frontal direction [0,0] found in HRTF.')
         _, h = hrtf.data[idx_frontal].tf(channels=0, n_bins=12, show=False)  # get low-res version of HRTF spectrum
@@ -297,18 +337,19 @@ class Binaural(Sound):
         resampled_signal = copy.deepcopy(self)
         # if sound and HRTF has different samplerates, resample the sound, apply the HRTF, and resample back:
         resampled_signal = resampled_signal.resample(hrtf.data[0].samplerate)  # resample to hrtf rate
-        filt = Filter(10**(h/20), fir=False, samplerate=hrtf.data[0].samplerate)
-        filtered_signal = filt.apply(resampled_signal)
-        filtered_signal = filtered_signal.resample(self.samplerate)
-        return filtered_signal
+        filt = Filter(10**(h/20), fir='TF', samplerate=hrtf.data[0].samplerate)
+        out = filt.apply(resampled_signal)
+        out = out.resample(self.samplerate)
+        out.name = f'externalized_{self.name}'
+        return out
 
     @staticmethod
     def make_interaural_level_spectrum(hrtf=None):
         """
-        Compute the frequency band specific interaural intensity differences for all sound source azimuth's in
+        Compute the frequency band specific binaural intensities for all sound source azimuth's in
         a head-related transfer function. For every azimuth in the hrtf, the respective transfer function is applied
-        to a sound. This sound is then divided into frequency sub-bands. The interaural level spectrum is the level
-        difference between right and left for each of these sub-bands for each azimuth.
+        to a sound. This sound is then divided into frequency sub-bands. The interaural level spectrum is the binaural
+        level difference at each sub-band and azimuth from the sub-bands of a sound at 0° azimuth.
         When individual HRTFs are not avilable, the level spectrum of the KEMAR mannequin may be used (default).
         Note that the computation may take a few minutes. Save the level spectrum to avoid re-computation, for instance
         with pickle or numpy.save (see documentation on readthedocs for examples).
@@ -317,10 +358,10 @@ class Binaural(Sound):
             hrtf (None | slab.HRTF): The head-related transfer function used to compute the level spectrum. If None,
                 use the recordings from the KEMAR mannequin.
         Returns:
-            (dict): A dictionary with keys `samplerate`, `frequencies` [n], `azimuths` [m], and `level_diffs` [n x m],
-                where `frequencies` lists the centres of sub-bands for which the level difference was computed, and
-                `azimuths` lists the sound source azimuth's in the hrft. `level_diffs` is a matrix of the interaural
-                level difference for each sub-band and azimuth.
+            (dict): A dictionary with keys `samplerate`, `frequencies` [n], `azimuths` [m], `level_diffs` [2, n, m],
+                where `frequencies` lists the centres of sub-bands for which the level difference was computed,
+                and `azimuths` lists the sound source azimuth's in the HRTF. `level_diffs` is a matrix of the level
+                spectrum for each ear, sub-band and azimuth.
         Examples::
 
             ils = slab.Binaural.make_interaural_level_spectrum()  # get the ils from the KEMAR recordings
@@ -331,28 +372,36 @@ class Binaural(Sound):
         """
         if not hrtf:
             hrtf = HRTF.kemar()  # load KEMAR by default
-        # get the filters for the frontal horizontal arc
-        idx = numpy.where((hrtf.sources[:, 1] == 0) & (
-            (hrtf.sources[:, 0] <= 90) | (hrtf.sources[:, 0] >= 270)))[0]
+            kemar_ils_path = pathlib.Path(__file__).parent.resolve() / pathlib.Path('data') / 'mit_kemar_ils.pkl'
+            with (open(kemar_ils_path, "rb")) as kemar_ils_file:
+                while True:
+                    try:
+                        return pickle.load(kemar_ils_file)
+                    except EOFError:
+                        break
+        # get the filters on the horizontal plane
+        idx = hrtf.get_source_idx(elevation=0)
         # at this point, we could just get the transfer function of each filter with hrtf.data[idx[i]].tf(),
         # but it may be better to get the spectral left/right differences with ERB-spaced frequency resolution:
-        azi = hrtf.sources[idx, 0]
-        # 270<azi<360 -> azi-360 to get negative angles on the left
-        azi[azi >= 270] = azi[azi >= 270]-360
+        azi = hrtf.sources.vertical_polar[idx, 0]
+        azi = ((azi + 180) % 360) - 180  # convert azimuth to half open interval [-180, 180]
         sort = numpy.argsort(azi)
         fbank = Filter.cos_filterbank(samplerate=hrtf.samplerate, pass_bands=True)
         freqs = fbank.filter_bank_center_freqs()
         noise = Sound.pinknoise(duration=5., samplerate=hrtf.samplerate)
+        noise_0 = Binaural(hrtf.data[idx[0]].apply(noise))
+        noise_0_bank = fbank.apply(noise_0.left)
         ils = dict()
         ils['samplerate'] = hrtf.samplerate
         ils['frequencies'] = freqs
         ils['azimuths'] = azi[sort]
-        ils['level_diffs'] = numpy.zeros((len(freqs), len(idx)))
+        ils['level_diffs'] = numpy.zeros((2, len(freqs), len(idx)))
         for n, i in enumerate(idx[sort]):  # put the level differences in order of increasing angle
             noise_filt = Binaural(hrtf.data[i].apply(noise))
             noise_bank_left = fbank.apply(noise_filt.left)
             noise_bank_right = fbank.apply(noise_filt.right)
-            ils['level_diffs'][:, n] = noise_bank_right.level - noise_bank_left.level
+            ils['level_diffs'][0, :, n] = noise_0_bank.level - noise_bank_left.level
+            ils['level_diffs'][1, :, n] = noise_0_bank.level - noise_bank_right.level
         return ils
 
     def interaural_level_spectrum(self, azimuth, ils=None):
@@ -380,31 +429,93 @@ class Binaural(Sound):
             ils = Binaural.make_interaural_level_spectrum()
         ils_samplerate = ils['samplerate']
         original_samplerate = self.samplerate
-        azis = ils['azimuths']
-        level_diffs = ils['level_diffs']
-        levels = numpy.array([numpy.interp(azimuth, azis, level_diffs[i, :]) for i in range(level_diffs.shape[0])])
+        level_diffs_left = ils['level_diffs'][0]
+        level_diffs_right = ils['level_diffs'][1]
+        levels_right = [numpy.interp(azimuth, ils['azimuths'], level_diffs_right[i, :]) for i in
+                        range(level_diffs_right.shape[0])]
+        levels_left = [numpy.interp(azimuth, ils['azimuths'], level_diffs_left[i, :]) for i in
+                       range(level_diffs_left.shape[0])]
         # resample the signal to the rate of the HRTF from which the filter was computed:
         resampled = self.resample(samplerate=ils_samplerate)
         fbank = Filter.cos_filterbank(length=resampled.n_samples, samplerate=ils_samplerate, pass_bands=True)
         subbands_left = fbank.apply(resampled.left)
         subbands_right = fbank.apply(resampled.right)
         # change subband levels:
-        subbands_left.level = subbands_left.level + levels / 2
-        subbands_right.level = subbands_right.level - levels / 2
+        subbands_left.level = subbands_left.level + levels_left
+        subbands_right.level = subbands_right.level + levels_right
         out_left = Filter.collapse_subbands(subbands_left, filter_bank=fbank)
         out_right = Filter.collapse_subbands(subbands_right, filter_bank=fbank)
         out = Binaural([out_left, out_right])
+        out.name = f'ils_{self.name}'
         return out.resample(samplerate=original_samplerate)
 
+    def drr(self, winlength=0.0025):
+        """
+        Calculate the direct-to-reverberant-ratio, DRR for the impulse input. This is calculated by
+        DRR = 10 * log10( X(T0-C:T0+C)^2 / X(T0+C+1:end)^2 ), where X is the approximated integral of the impulse,
+        T0 is the time of the direct impulse, and C=2.5ms (Zahorik, P., 2002: 'Direct-to-reverberant energy ratio
+        sensitivity', The Journal of the Acoustical Society of America, 112, 2110-2117)
+
+        Arguments:
+            winlength (int | float): specifies the length of the direct sound window. This window is used to calculate
+            the energy of impulse sound, starting from the position of the peak amplitude of the impulse.
+
+        Returns:
+            Direct-to-reverberation value of the input impulse measured in dB
+
+        Return type:
+            (float)
+        """
+        # convert winlength parameter to samples
+        winlength = Sound.in_samples(winlength, self.samplerate)
+        # winlength should take minimum 2 sample values
+        winlength = max(2, winlength)
+        if winlength > len(self.data):
+            raise ValueError("'winlength' should be shorter than input sound.")
+        elif winlength > 0.01 * self.samplerate:
+            raise ValueError("'winlength' is suggested to be under 10ms.")
+        if self.samplerate < 5000:
+            raise ValueError("Sampling frequency is too low, it should be at least 5000 Hz.")
+        correction = round(0.5 * self.samplerate / 1000 / 1000) # safety margin of 0.5ms
+        # calculate drr for left channel
+        impulse = self.data[:, 0]
+        impulse_sq = numpy.square(impulse)
+        # find the location (index) of the maximal peak in the impulse signal
+        peak_index = impulse_sq.argmax()
+        # calculate direct and reverberant energy values by integrating under their RMS curve
+        win_start_index = max(0, peak_index - correction)
+        win_end_index = peak_index + winlength
+        direct = impulse[win_start_index: win_end_index]
+        direct_e = numpy.trapz(numpy.square(direct))
+        reverb = impulse[win_end_index + 1:]
+        reverb_e = numpy.trapz(numpy.square(reverb))
+        # Calculate DRR as a value in dB
+        if direct_e == 0:
+            raise ValueError("Direct energy is 0. Please check that your input parameters are reasonable.")
+        elif reverb_e == 0:
+            raise ValueError("Reverb energy is 0. Are you sure this is an impulse?")
+        elif reverb_e < sys.float_info.min:
+            raise ValueError("Reverb energy is too low. Please check that your input parameters are reasonable.")
+        ratio = direct_e / reverb_e
+        drr = 10 * math.log10(ratio)
+        # Return output
+        return drr
+
     @staticmethod
     def whitenoise(kind='diotic', **kwargs):
         """
         Generate binaural white noise. `kind`='diotic' produces the same noise samples in both channels,
         `kind`='dichotic' produces uncorrelated noise. The rest is identical to `slab.Sound.whitenoise`.
         """
-        out = Binaural(Sound.whitenoise(**kwargs))
-        if kind == 'diotic':
+        if kind == 'dichotic':
+            out = Binaural(Sound.whitenoise(n_channels=2, **kwargs))
+            out.left = Sound.whitenoise(n_channels=1, **kwargs)
+        elif kind == 'diotic':
+            out = Binaural(Sound.whitenoise(n_channels=2, **kwargs))
             out.left = out.right
+        else:
+            raise ValueError("kind must be 'dichotic' or 'diotic'.")
+        out.name = f'{kind}-{out.name}'
         return out
 
     @staticmethod
@@ -413,7 +524,9 @@ class Binaural(Sound):
         Generate binaural pink noise. `kind`='diotic' produces the same noise samples in both channels,
         `kind`='dichotic' produces uncorrelated noise. The rest is identical to `slab.Sound.pinknoise`.
         """
-        return Binaural.powerlawnoise(alpha=1, **kwargs)
+        out = Binaural.powerlawnoise(alpha=1, kind=kind, **kwargs)
+        out.name = f'{kind}-pinknoise'
+        return out
 
     @staticmethod
     def powerlawnoise(kind='diotic', **kwargs):
@@ -429,6 +542,7 @@ class Binaural(Sound):
             out.left = out.right
         else:
             raise ValueError("kind must be 'dichotic' or 'diotic'.")
+        out.name = f'{kind}-{out.name}'
         return out
 
     @staticmethod
@@ -442,6 +556,7 @@ class Binaural(Sound):
             out.left = out.right
         else:
             raise ValueError("kind must be 'dichotic' or 'diotic'.")
+        out.name = f'{kind}-{out.name}'
         return out
 
     @staticmethod
