@@ -16,8 +16,12 @@ from slab.signal import Signal  # getting the base class
 
 class Filter(Signal):
     """
-    Class for generating and manipulating filter banks and transfer functions. Filters can be either finite impulse
-    response (FIR) or Fourier filters.
+    Class for generating and manipulating filter banks and transfer functions. Filters can be finite impulse
+    response ('FIR'), impulse response ('IR'), or transfer functions ('TF'). FIR filters are applied using
+    two-way filtering (see scipy.signal.filtfilt), which avoids adding group delays and is intended for high-
+    and lowpass filters and the like. IR filters are applied using convolution (scipy.signal.fftconvolve) and
+    are intended for long room impulse responses, binaural impulse responses and the like, where group delays
+    are intended.
 
     Arguments:
         data (numpy.ndarray | slab.Signal | list): samples of the filter. If it is an array, the first dimension
@@ -25,7 +29,7 @@ class Filter(Signal):
             it must have a .data attribute containing an array. If it's a list, the elements can be arrays or objects.
             The output will be a multi-channel sound with each channel corresponding to an element of the list.
         samplerate (int | None): the samplerate of the sound. If None, use the default samplerate.
-        fir (bool): whether this is a finite impulse filter (True) or a Fourier filter (False),
+        fir (str): the kind of filter; options are 'FIR', 'IR', or 'TF'.
     Attributes:
         .n_filters: number of filters in the object (overloads `n_channels` attribute in the parent `Signal` class)
         .n_taps: the number of taps in a finite impulse response filter. Analogous to `n_samples` in a `Signal`.
@@ -37,24 +41,28 @@ class Filter(Signal):
     n_taps = property(fget=lambda self: self.n_samples, doc='The number of filter taps.')
     n_frequencies = property(fget=lambda self: self.n_samples, doc='The number of frequency bins.')
     frequencies = property(fget=lambda self: numpy.fft.rfftfreq(self.n_frequencies * 2 - 1, d=1 / self.samplerate)
-                           if not self.fir else None, doc='The frequency axis of the filter.')
+                           if self.fir=='TF' else None, doc='The frequency axis of the filter.')
 
-    def __init__(self, data, samplerate=None, fir=True):
-        if fir and scipy is False:
-            raise ImportError('FIR filters require scipy.')
+    def __init__(self, data, samplerate=None, fir='FIR'):
+        if isinstance(fir, bool):
+            raise AttributeError("Assigning a boolean to fir is deprecated. Use 'TF' instead of False, and 'FIR' or 'IR' otherwise.")
+        if 'IR' in fir and scipy is False:
+            raise ImportError('(F)IR filters require scipy.')
         super().__init__(data, samplerate)
+        if not (fir=='FIR' or fir=='IR' or fir=='TF'):
+            raise ValueError("fir must be 'FIR', 'IR', or 'TF'!")
         self.fir = fir
 
     def __repr__(self):
         return f'{type(self)} (\n{repr(self.data)}\n{repr(self.samplerate)}\n{repr(self.fir)})'
 
     def __str__(self):
-        if self.fir:
+        if 'IR' in self.fir:
             return f'{type(self)}, filters {self.n_filters}, FIR: taps {self.n_taps}, samplerate {self.samplerate}'
-        return f'{type(self)}, filters {self.n_filters}, FFT: freqs {self.n_frequencies}, samplerate {self.samplerate}'
+        return f'{type(self)}, filters {self.n_filters}, TF: freqs {self.n_frequencies}, samplerate {self.samplerate}'
 
     @staticmethod
-    def band(kind='hp', frequency=100, gain=None, samplerate=None, length=1000, fir=True):
+    def band(kind='hp', frequency=100, gain=None, samplerate=None, length=1000, fir='FIR'):
         """
         Generate simple passband or stopband filters, or filters with a transfer function defined by  pairs
         of `frequency` and `gain` values.
@@ -71,7 +79,7 @@ class Filter(Signal):
                 1.0 (no suppression at that frequency) and 0.0 (maximal suppression at that frequency).
             samplerate (int | None): the samplerate of the sound. If None, use the default samplerate.
             length (int): The number of samples in the filter
-            fir: If true generate a finite impulse response filter, else generate a Fourier filter.
+            fir (str): If 'FIR' or 'IR' generate a finite impulse response filter, else generate a transfer function.
         Returns:
             (slab.Filter): a filter with the specified properties
         Examples::
@@ -82,7 +90,7 @@ class Filter(Signal):
         """
         if samplerate is None:
             samplerate = slab.signal._default_samplerate
-        if fir:  # design a FIR filter
+        if 'IR' in fir:  # design a FIR filter
             if scipy is False:
                 raise ImportError('Generating FIR filters requires Scipy.')
             if gain is None:  # design band filter
@@ -107,7 +115,7 @@ class Filter(Signal):
                     nyq_gain = [0]
                 filt = scipy.signal.firwin2(numtaps=length, freq=dc+frequency+nyq, gain=dc_gain+gain+nyq_gain,
                                             fs=samplerate)
-        else:  # FFR filter
+        else:  # TF filter
             df = (samplerate/2) / (length-1)
             if gain is None:
                 filt = numpy.zeros(length)
@@ -135,6 +143,10 @@ class Filter(Signal):
         In that case the filtered sound wil contain the same number of channels as the filter with every
         channel being a copy of the original sound with one filter channel applied. If the filter has only
         one channel and the sound has multiple channels, the same filter is applied to each sound channel.
+        FIR filters are applied using two-way filtering (see scipy.signal.filtfilt), which avoids adding
+        group delays and is intended for high- and lowpass filters and the like. IR filters are applied
+        using convolution (scipy.signal.fftconvolve) and are intended for long room impulse responses,
+        binaural impulse responses and the like, where group delays are intended.
 
         Arguments:
             sig (slab.Signal | slab.Sound): The sound to be filtered.
@@ -149,22 +161,43 @@ class Filter(Signal):
         if (self.samplerate != sig.samplerate) and (self.samplerate != 1):
             raise ValueError('Filter and sound have different sampling rates.')
         out = copy.deepcopy(sig)
-        if self.fir:
+        if self.fir == 'FIR':
             if scipy is False:
                 raise ImportError('Applying FIR filters requires Scipy.')
+            if out.n_samples < self.n_samples * 3:
+                padlen = out.n_samples - 1
+            else:
+                padlen = None
             if self.n_filters == sig.n_channels:  # filter each channel with corresponding filter
                 for i in range(self.n_filters):
                     out.data[:, i] = scipy.signal.filtfilt(
-                        self.data[:, i], [1], out.data[:, i], axis=0)
+                        self.data[:, i], [1], out.data[:, i], axis=0, padlen=padlen)
             elif (self.n_filters == 1) and (sig.n_channels > 1):  # filter each channel
-                for i in range(self.n_filters):
+                for i in range(sig.n_channels):
                     out.data[:, i] = scipy.signal.filtfilt(
-                        self.data.flatten(), [1], out.data[:, i], axis=0)
+                        self.data.flatten(), [1], out.data[:, i], axis=0,  padlen=padlen)
             elif (self.n_filters > 1) and (sig.n_channels == 1):  # apply all filters in bank to sound
                 out.data = numpy.empty((sig.n_samples, self.n_filters))
                 for filt in range(self.n_filters):
                     out.data[:, filt] = scipy.signal.filtfilt(
-                        self[:, filt], [1], sig.data, axis=0).flatten()
+                        self[:, filt], [1], sig.data, axis=0,  padlen=padlen).flatten()
+            else:
+                raise ValueError(
+                    'Number of filters must equal number of sound channels, or either one of them must be equal to 1.')
+        elif self.fir == 'IR':
+            length = sig.n_samples + self.n_taps - 1
+            if self.n_filters == sig.n_channels:  # filter each channel with corresponding filter
+                out.data = numpy.empty((length, sig.n_channels))
+                for i in range(self.n_filters):
+                    out.data[:, i] = scipy.signal.fftconvolve(sig[:, i], self[:, i], mode='full')
+            elif (self.n_filters == 1) and (sig.n_channels > 1):  # filter each channel
+                out.data = numpy.empty((length, sig.n_channels))
+                for i in range(sig.n_channels):
+                    out.data[:, i] = scipy.signal.fftconvolve(sig[:, i], self[:, 0], mode='full')
+            elif (self.n_filters > 1) and (sig.n_channels == 1):  # apply all filters in bank to sound
+                out.data = numpy.empty((length, self.n_filters))
+                for filt in range(self.n_filters):
+                    out.data[:, filt] = scipy.signal.fftconvolve(sig[:, 0], self[:, filt], mode='full').flatten()
             else:
                 raise ValueError(
                     'Number of filters must equal number of sound channels, or either one of them must be equal to 1.')
@@ -229,7 +262,7 @@ class Filter(Signal):
             channels = list(range(self.n_filters))  # now we have a list of filter indices to process
         if n_bins is None:
             n_bins = self.data.shape[0]
-        if self.fir:
+        if 'IR' in self.fir:
             if scipy is False:
                 raise ImportError('Computing transfer functions of FIR filters requires Scipy.')
             h = numpy.empty((n_bins, len(channels)))
@@ -263,25 +296,28 @@ class Filter(Signal):
             return w, h
 
     @staticmethod
-    # TODO: oversampling factor needed for cochleagram!
-    def cos_filterbank(length=5000, bandwidth=1/3, low_cutoff=0, high_cutoff=None, pass_bands=False, samplerate=None):
+    def cos_filterbank(length=5000, bandwidth=1/3, low_cutoff=0, high_cutoff=None, pass_bands=False, n_filters=None,
+                       filter_width_factor=1, jitter=False, samplerate=None):
         """
         Generate a set of Fourier filters. Each filter's transfer function is given by the positive phase of a
         cosine wave. The amplitude of the cosine is that filters central frequency. Following the organization of the
         cochlea, the width of the filter increases in proportion to it's center frequency. This increase is defined
-        by Moore & Glasberg's formula for the equivalent rectangular bandwidth (ERB) of auditory filters. This
-        functions is used for example to divide a sound into bands for equalization.
+        by Moore & Glasberg's formula for the equivalent rectangular bandwidth (ERB) of auditory filters.
+        The number of filters is either determined by the `n_filters` argument or calculated based on the desired
+        `bandwidth` or. This function is used for example to divide a sound into sub-bands for equalization.
 
         Attributes:
             length (int): The number of bins in each filter, determines the frequency resolution.
-            bandwidth (float): Width of the sub-filters in octaves. The smaller the bandwidth, the more filters
-                will be generated.
+            bandwidth (float): Width of the sub-filters in octaves. The smaller the bandwidth, the more filters will be generated.
             low_cutoff (int | float): The lower limit of frequency range in Hz.
             high_cutoff (int | float): The upper limit of frequency range in Hz. If None, use the Nyquist frequency.
             pass_bands (bool): Whether to include a half cosine at the filter bank's lower and upper edge frequency.
                 If True, allows reconstruction of original bandwidth when collapsing subbands.
+            n_filters (int | None): Number of filters. When this is not None, the `bandwidth` argument is ignored.
+            filter_width_factor (float): Multiplier for the width of the filters. Default is 1; use smaller values to make the filter coverage sparser (undersampled) and larger values to make it denser (oversampled).
+            jitter (bool): if True and filter_width_factor <1, jitter the center frequencies within the original band.
             samplerate (int | None): the samplerate of the sound that the filter shall be applied to.
-                If None, use the default samplerate.s
+                If None, use the default samplerate.
         Examples::
 
             sig = slab.Sound.pinknoise(samplerate=44100)
@@ -300,27 +336,35 @@ class Filter(Signal):
         freq_bins = numpy.fft.rfftfreq(length, d=1/samplerate)
         n_freqs = len(freq_bins)
         center_freqs, bandwidth, erb_spacing = Filter._center_freqs(
-            low_cutoff=low_cutoff, high_cutoff=high_cutoff, bandwidth=bandwidth, pass_bands=pass_bands)
+            low_cutoff=low_cutoff, high_cutoff=high_cutoff, bandwidth=bandwidth, pass_bands=pass_bands, n_filters=n_filters)
+        erb_spacing_adjusted = erb_spacing * filter_width_factor
         n_filters = len(center_freqs)
         filts = numpy.zeros((n_freqs, n_filters))
         freqs_erb = Filter._freq2erb(freq_bins)
+        jitter_amp = erb_spacing * (1-filter_width_factor) # erb_spacing minus adjusted = jitter amplitude
+        width = erb_spacing_adjusted * 2 # width of filters
         for i in range(n_filters):
-            l = center_freqs[i] - erb_spacing
-            h = center_freqs[i] + erb_spacing
+            if jitter and filter_width_factor<1:
+                center_freqs[i] = center_freqs[i] + numpy.random.uniform(low=-jitter_amp, high=jitter_amp)
+            l = center_freqs[i] - erb_spacing_adjusted
+            h = center_freqs[i] + erb_spacing_adjusted
             avg = center_freqs[i]  # center of filter
-            width = erb_spacing * 2  # width of filter
             filts[(freqs_erb > l) & (freqs_erb < h), i] = numpy.cos(
                 (freqs_erb[(freqs_erb > l) & (freqs_erb < h)] - avg) / width * numpy.pi)
-        return Filter(data=filts, samplerate=samplerate, fir=False)
+        return Filter(data=filts, samplerate=samplerate, fir='TF')
 
     @staticmethod
-    def _center_freqs(low_cutoff, high_cutoff, bandwidth=1/3, pass_bands=False):
+    def _center_freqs(low_cutoff, high_cutoff, bandwidth=1/3, pass_bands=False, n_filters=None):
         ref_freq = 1000  # Hz, reference for conversion between oct and erb bandwidth
-        ref_erb = Filter._freq2erb(ref_freq)
-        erb_spacing = Filter._freq2erb(ref_freq*2**bandwidth) - ref_erb
         h = Filter._freq2erb(high_cutoff)
         l = Filter._freq2erb(low_cutoff)
-        n_filters = int(numpy.round((h - l) / erb_spacing))
+        ref_erb = Filter._freq2erb(ref_freq)
+        if n_filters is None:
+            erb_spacing = Filter._freq2erb(ref_freq*2**bandwidth) - ref_erb
+            n_filters = int(numpy.round((h - l) / erb_spacing))
+        elif n_filters is not None and pass_bands is False:
+            # add 2 so that after omitting pass_bands we get the desired n_filt
+            n_filters += 2
         center_freqs, erb_spacing = numpy.linspace(l, h, n_filters, retstep=True)
         if not pass_bands:
             center_freqs = center_freqs[1:-1]  # exclude low and highpass filters
@@ -357,7 +401,7 @@ class Filter(Signal):
         if not filter_bank:
             filter_bank = Filter.cos_filterbank(
                 length=subbands.n_samples, samplerate=subbands.samplerate)
-        elif filter_bank.fir:
+        elif 'IR' in filter_bank.fir:
             raise ValueError("Not implemented for FIR filters!")
         if subbands.samplerate != filter_bank.samplerate:
             raise ValueError('Signal and filter bank need to have the same samplerate!')
@@ -376,7 +420,7 @@ class Filter(Signal):
             (numpy.ndarray): array with length equal to the number of filters in the bank, containing each filter's
                 center frequency.
         """
-        if self.fir:
+        if 'IR' in self.fir:
             raise NotImplementedError('Not implemented for FIR filter banks.')
         freqs = self.frequencies
         center_freqs = numpy.zeros(self.n_filters)
@@ -387,7 +431,7 @@ class Filter(Signal):
 
     @staticmethod
     def equalizing_filterbank(reference, sound, length=1000, bandwidth=1/8, low_cutoff=200, high_cutoff=None,
-                              alpha=1.0):
+                              alpha=1.0, filt_meth='filtfilt'):
         """
         Generate an equalizing filter from the spectral difference between a `sound` and a `reference`. Both are
         divided into sub-bands using the `cos_filterbank` and the level difference per sub-band is calculated. The
@@ -406,6 +450,10 @@ class Filter(Signal):
             high_cutoff (int | float): The upper limit of frequency range in Hz. If None, use the Nyquist frequency.
             alpha (float): Filter regularization parameter. Values below 1.0 reduce the filter's effect, values above
                 amplify it. WARNING: large filter gains may result in temporal distortions of the sound
+            filt_meth (str): filtering method, must be in ('filtfilt', 'fft', 'slab', 'causal'). 'filtfilt' and 'slab'
+                applies the filter twice (forward and backward) so the filtered signal is not time-shifted, but in this
+                case we need a filter with half of the desired amplitude so the end result has the correct modification.
+                For 'fft' and 'causal', the filter with desired amplitude is returned
         Returns:
             (slab.Filter): An equalizing filter bank with a number of filters equal to the number of channels in the
                 equalized `sound`.
@@ -418,10 +466,12 @@ class Filter(Signal):
             filtered = filt.apply(sound)
             # make an equalizing filter and apply it to the filtered signal. The result looks more like the original
             fbank = slab.Filter.equalizing_filterbank(sound, filtered, low_cutoff=200, high_cutoff=16000)
-            equalized = fbank.apply(sound)
+            equalized = fbank.apply(filtered)
         """
         if scipy is False:
             raise ImportError('Generating equalizing filter banks requires Scipy.')
+        if filt_meth not in ('filtfilt', 'fft', 'slab', 'causal'):
+            raise ValueError("filt_meth must be one of {'filtfilt', 'fft', 'slab', 'causal'}")
         if reference.n_channels > 1:
             raise ValueError("The reference sound must have only one channel!")
         if bool(reference.n_samples % 2):  # number of samples must be even:
@@ -434,6 +484,14 @@ class Filter(Signal):
             sound = sound.resample(reference.samplerate)
         if high_cutoff is None:
             high_cutoff = reference.samplerate / 2
+        # different conditions on filtering method
+        is_fir = 'FIR'
+        filt_scaling = 1
+        if filt_meth == 'fft':
+            is_fir = 'TF'
+        elif filt_meth in ('slab', 'filtfilt'):
+            filt_scaling = 0.5
+
         fbank = Filter.cos_filterbank(length=length, bandwidth=bandwidth,
                                       low_cutoff=low_cutoff, high_cutoff=high_cutoff, samplerate=reference.samplerate)
         center_freqs, _, _ = Filter._center_freqs(low_cutoff, high_cutoff, bandwidth)
@@ -447,19 +505,19 @@ class Filter(Signal):
         levels_signal = numpy.ones((len(center_freqs), sound.n_channels))
         for idx in range(sound.n_channels):
             levels_signal[:, idx] = fbank.apply(sound.channel(idx)).level
-        amp_diffs = levels_target - levels_signal
-        max_diffs = numpy.max(numpy.abs(amp_diffs), axis=0)
-        max_diffs[max_diffs == 0] = 1
-        amp_diffs = amp_diffs/max_diffs
-        amp_diffs *= alpha  # apply factor for filter regulation
-        amp_diffs += 1  # add 1 because gain = 1 means "do nothing"
+        amp_diffs_dB = levels_target - levels_signal
+        # if we want to keep the scaling
+        amp_diffs_dB = filt_scaling * amp_diffs_dB * alpha
+        # convert differences from dB to ratio
+        amp_diffs = 10 ** (amp_diffs_dB / 20.)
         center_freqs = [0] + center_freqs
         filts = numpy.zeros((length, sound.n_channels))
         for chan in range(sound.n_channels):
             gain = [1] + list(amp_diffs[:, chan])
-            filt = Filter.band(frequency=list(center_freqs), gain=gain, samplerate=reference.samplerate, fir=True)
+            filt = Filter.band(frequency=list(center_freqs), gain=gain, samplerate=reference.samplerate, fir=is_fir,
+                               length=length)
             filts[:, chan] = filt.data.flatten()
-        return Filter(data=filts, samplerate=reference.samplerate, fir=True)
+        return Filter(data=filts, samplerate=reference.samplerate, fir=is_fir)
 
     def save(self, filename):
         """
@@ -468,7 +526,13 @@ class Filter(Signal):
             filename(str | pathlib.Path): Full path to which the data is saved.
         """
         fs = numpy.tile(self.samplerate, reps=self.n_filters)
-        fir = numpy.tile(self.fir, reps=self.n_filters)
+        if self.fir == 'IR':
+            fir_coded = 1
+        elif self.fir == 'TF':
+            fir_coded = 2
+        else:
+            fir_coded = 0
+        fir = numpy.tile(fir_coded, reps=self.n_filters)
         fs = fs[numpy.newaxis, :]
         fir = fir[numpy.newaxis, :]
         to_save = numpy.concatenate((fs, fir, self.data))  # prepend the samplerate as new 'filter'
@@ -486,7 +550,13 @@ class Filter(Signal):
         """
         data = numpy.load(filename)
         samplerate = data[0][0]  # samplerate is in the first filter
-        fir = bool(data[1][0])  # fir is in the first filter
+        fir_coded = data[1][0]  # fir is in the first filter
+        if fir_coded == 2:
+            fir = 'TF'
+        elif fir_coded == 1:
+            fir = 'IR'
+        else:
+            fir = 'FIR'
         data = data[2:, :]  # drop the samplerate and fir entries
         return Filter(data, samplerate=samplerate, fir=fir)
 
