@@ -24,21 +24,20 @@ except ImportError:
 import slab
 
 results_folder = 'Results'
-input_method = 'keyboard'  #: sets the input for the Key context manager to 'keyboard 'or 'buttonbox'
+input_method = 'keyboard'  #: sets the input for the Key context manager to 'keyboard', 'buttonbox', 'prompt', or 'figure'
 
 
 class _Buttonbox:
     """
-    Adapter class to allow easy switching between input from the keyboard via curses and from the custom buttonbox
-    adapter (custom arduino device that sends a keystroke followed by a return keystroke when pressing a button on
-    the arduino).
+    Adapter class to allow easy switching between input from the keyboard via curses (no ENTER keypress needed after
+    single button press) to the Python input prompt, which can be used with an external button box (custom arduino device
+    that sends keystrokes) or when running from a Jupiter or Colab notebook.
     """
     @staticmethod
     def getch():
-        input_key = input()  # buttonbox adapter has to return the keycode of intended keys!
+        input_key = input()  # buttonbox has to return the key followed by ENTER!
         if input_key:
-            return int(input_key)
-
+            return ord(input_key)
 
 class _FigChar:
     """
@@ -47,7 +46,7 @@ class _FigChar:
     already present. If used together with the plot method of the Staircase class, input is acquired through the stairs
     plot. Depending on the operating system, you may have to click once into the figure to give it focus.
     """
-    warnings.filterwarnings("ignore", category=matplotlib.cbook.MatplotlibDeprecationWarning)
+    warnings.filterwarnings("ignore", category=matplotlib.MatplotlibDeprecationWarning)
 
     @staticmethod
     def getch():
@@ -67,9 +66,10 @@ class _FigChar:
 def key(mesg=None):
     """
     Wrapper for curses module to simplify getting a single keypress from the terminal (default), a buttonbox, or a
-    figure. Set slab.psychoacoustics.input_method = 'buttonbox' to use a custom USB buttonbox or to 'figure' to open
-    a figure called 'stairs' (if not already opened by the `slab.Staricase.plot` method). Optianally takes a string
-    argument which is printed in the terminal for conveying instructions to the participant.
+    figure. Set slab.psychoacoustics.input_method = 'buttonbox' to use a custom USB buttonbox, to 'figure' to open
+    a figure called 'stairs' (if not already opened by the `slab.Staricase.plot` method), or to 'prompt' for a
+    simple Python prompt (requires pressing Enter/Return after the response). Optionally takes a string argument
+    which is printed in the terminal for conveying instructions to the participant.
 
     Example::
 
@@ -93,7 +93,7 @@ def key(mesg=None):
         curses.nocbreak()
         curses.echo()
         curses.endwin()
-    elif input_method == 'buttonbox':
+    elif input_method == 'buttonbox' or input_method == 'prompt':
         if mesg is not None:
             print(mesg)
         yield _Buttonbox
@@ -153,7 +153,7 @@ class LoadSaveMixin:
         Returns:
             (bool): True if writing was successful.
         """
-        def default(i): return int(i) if isinstance(i, numpy.int64) else i  # helper for converting numpy arrays
+        def default(i): return int(i) if isinstance(i, (numpy.int64,numpy.int32)) else i  # helper for converting numpy arrays
         if isinstance(file_name, pathlib.Path):
             file_name = str(file_name)
         if (file_name is None) or (file_name == 'stdout'):
@@ -244,7 +244,7 @@ class TrialPresentationOptionsMixin:
         stimulus.play()
         with slab.key() as k:
             response = k.getch()
-        response = response in [key_codes[i] for i in correct_key_idx]
+        response = response == key_codes[correct_key_idx]
         self.add_response(response)
         if print_info:
             self.print_trial_info()
@@ -426,9 +426,9 @@ class Trialsequence(collections.abc.Iterator, LoadSaveMixin, TrialPresentationOp
             self.this_trial = self.conditions[self.trials[self.this_n]-1]  # fetch the trial info
         return self.this_trial
 
-    def add_response(self, response):
+    def add_response(self, *response):
         """
-        Append response to the list in the `data` attribute belonging to the current trial (see Trialsequence doc).
+        Append response(s) to the list in the `data` attribute belonging to the current trial (see Trialsequence doc).
 
         Attributes:
              response (any): data to append to the list. Can be anything but save_json method won't be available if
@@ -437,7 +437,8 @@ class Trialsequence(collections.abc.Iterator, LoadSaveMixin, TrialPresentationOp
         if self.this_n < 0:
             print("Can't add response because trial hasn't started yet!")
         else:
-            self.data[self.this_n].append(response)
+            for r in response:
+                self.data[self.this_n].append(r)
 
     def print_trial_info(self):
         """ Convenience method for printing current trial information. """
@@ -650,8 +651,8 @@ class Staircase(collections.abc.Iterator, LoadSaveMixin, TrialPresentationOption
         n_down (int): number of `correct` (or 1) responses before the staircase level decreases (see `n_up`).
         step_type (str): defines the change of stimulus intensity at each step of the staircase. possible inputs are
             'lin' (adds or subtract a certain amount), 'db', and 'log' (prevents the intensity from reaching zero).
-        min_val (int or float): smallest stimulus value permitted, or -Inf for staircase without lower limit
-        max_val (int or float): largest stimulus value permitted, or Inf for staircase without upper limit
+        min_val (int or float): smallest stimulus value permitted, or -inf for staircase without lower limit
+        max_val (int or float): largest stimulus value permitted, or inf for staircase without upper limit
         label (str): text label for the sequence, defaults to an empty string
     Attributes:
         .this_trial_n: number of completed trials
@@ -673,7 +674,7 @@ class Staircase(collections.abc.Iterator, LoadSaveMixin, TrialPresentationOption
         print(f'mean of final 6 reversals: {stairs.threshold()}')
     """
     def __init__(self, start_val, n_reversals=None, step_sizes=1, step_up_factor=1, n_pretrials=0, n_up=1,
-                 n_down=2, step_type='lin', min_val=-numpy.Inf, max_val=numpy.Inf, label=''):
+                 n_down=2, step_type='lin', min_val=-numpy.inf, max_val=numpy.inf, label=''):
         self.label = label
         self.start_val = start_val
         self.n_up = n_up
@@ -899,8 +900,8 @@ class Staircase(collections.abc.Iterator, LoadSaveMixin, TrialPresentationOption
             axis.clear()
             axis.plot(x, y)
             axis.set_xlim(-self.n_pretrials, max(20, (self.this_trial_n + 15)//10*10))
-            axis.set_ylim(min(0, min(y)) if self.min_val == -numpy.Inf else self.min_val,
-                          max(y) if self.max_val == numpy.Inf else self.max_val)
+            axis.set_ylim(min(0, min(y)) if self.min_val == -numpy.inf else self.min_val,
+                          max(y) if self.max_val == numpy.inf else self.max_val)
             # plot green dots at correct/yes responses
             axis.scatter(x[responses], y[responses], color='green')
             # plot red dots at correct/yes responses
@@ -952,7 +953,7 @@ class ResultsFile:
 
     Arguments:
         subject (str): determines the name of the sub-folder and files.
-        folder (None | str): folder in which all results are saved, if None use the global variable results_folder.
+        folder (None | str): folder in which all results are saved, defaults to global variable results_folder.
     Attributes:
         .path: full path to the results file.
         .subject: the subject's name.
@@ -965,23 +966,24 @@ class ResultsFile:
 
     name = property(fget=lambda self: self.path.name, doc='The name of the results file.')
 
-    def __init__(self, subject='test', folder=None):
+    def __init__(self, subject='test', folder=None, filename=None):
         self.subject = subject
         if folder is None:
             folder = results_folder
-        self.path = pathlib.Path(folder / pathlib.Path(subject) / pathlib.Path(subject +
+        filename = '_'.join(filter(None, (subject, filename)))
+        self.path = pathlib.Path(folder / pathlib.Path(subject) / pathlib.Path(filename +
                                  datetime.datetime.now().strftime("_%Y-%m-%d-%H-%M-%S") + '.txt'))
         # make the Results folder and subject subfolder
         self.path.parent.mkdir(parents=True, exist_ok=True)
 
     def write(self, data, tag=None):
         """
-        Safely write data to the file which is opened just before writing and closed immediately after to avoid
-        data loss. Call this method at the end of each trial to save the response and trial state.
+        Safely write data to the file which is opened just before writing and closed immediately after
+        to avoid data loss. Call this method at the end of each trial to save the response and trial state.
 
         Arguments:
-            data (any): data to save must be JSON serializable [string, list, dict, ...]). If data is an object,
-                the __dict__ is extracted and saved.
+            data (any): data to save must be JSON serializable [string, list, dict, ...]). If data is an
+                object, the __dict__ is extracted and saved.
             tag (str): The tag is prepended as a key. If None is provided, the current time is used.
         """
         if hasattr(data, "__dict__"):
@@ -1005,9 +1007,9 @@ class ResultsFile:
             filename (str | pathlib.Path):
             tag (None | str):
         Returns:
-            (list | dict): The content of the file. If tag is None, the whole file is returned, else only the
-                dictionaries with that tag as a key are returned. The content will be a list of dictionaries or a
-                dictionary if there is only a single element.
+            (list | dict): The content of the file. If tag is None, the whole file is returned,
+                else only the dictionaries with that tag as a key are returned. The content will
+                be a list of dictionaries or a dictionary if there is only a single element.
         """
         content = []
         with open(filename) as file:
@@ -1049,6 +1051,94 @@ class ResultsFile:
             file.write('')
 
 
+class ResultsTable(ResultsFile):
+    """
+    A class for foolproof writing of results tables as comma separated values (CSV), including
+    generating the name, creating the folders, and writing to the file after each trial. On
+    initialization you have to provide the column headers of the CSV table, either as a list,
+    a comma_separated string, or as Path object to a separate text file containing the column
+    headers.
+    The ResultsTable object
+
+    Arguments:
+        subject (str): determines the name of the sub-folder and files.
+        columns (list | str | path.Path): list of column names or comma-separated column names
+            in a string (read from a text file if Path object is given). Must be valid Python variable
+            names! Will be the first row of the results file. Adding rows to the table requires giving
+            a value for each variable name. (This is enforced through a namedtuple of these variables.)
+        folder (None | str): folder in which all results are saved, defaults to global variable results_folder.
+    Attributes:
+        .path: full path to the results file.
+        .subject: the subject's name.
+        .name: file name
+        .Row: Namedtuple with column names as fields. Must be fully populated to write to the table.
+    Example::
+
+        ResultsTable.results_folder = 'MyResults'
+        header = 'timestamp, subject, trial, stimulus, response'
+        # OR: header = ('timestamp', 'subject', etc.)
+        # OR: header = path.Path('header_names.csv')
+        table = ResultsTable(subject='MS', columns=header)
+        print(table.name) # this file is now created and contains the header row
+        print(table.Row) # a namedtuple has also been created with the header attributes
+        print(table.Row._fields) # the fields are the column names
+        # to write a row of results at the end of the trial loop:
+        row = table.Row(timestamp=datetime.now(), subject=table.subject, trail=stairs.this_n, stimulus=stim.name, response=button)
+        table.write(row)
+    """
+
+    def __init__(self, columns, subject='test', folder=None, filename=None):
+        super().__init__(subject, folder, filename)
+        self.Row = self._make_Row(columns)
+        self._write_header()
+
+    @staticmethod
+    def _make_Row(columns):
+        "Generate a namedtuple with fields corresponding to column names. Called automatically during init."
+        if isinstance(columns, pathlib.Path):
+            with open(columns) as f:
+                first_line = f.readline().strip('\n')
+                field_names = [x.strip() for x in first_line.split(',')]
+        elif isinstance(columns, str):
+            field_names = [x.strip() for x in columns.split(',')]
+        else:
+            field_names = columns
+        return collections.namedtuple('Row', field_names)
+
+    def write(self, row):
+        """
+        Safely write data to the file which is opened just before writing and closed immediately
+        after to avoid data loss. Call this method at the end of each trial to save the response
+        and trial state. Values with commas are enclosed in double quotes to keep the CSV valid.
+
+        Arguments:
+            row (self.Row): All values to be written have to be provided as namedtuple, the fields
+                of which were generated at initialization time and cannot be changed. This ensures
+                table consistency.
+        Example::
+            # to write a row of results at the end of the trial loop, first make a new instance of Row:
+            row = table.Row(timestamp=datetime.now(), subject=table.subject, trail=stairs.this_n, stimulus=stim.name, response=button)
+            # then write these row values to the file:
+            table.write(row)
+        """
+        if not isinstance(row, self.Row):
+            raise TypeError('Data has to be given as instance of namedtuple Row.')
+        with open(self.path, 'a') as file:
+            vals = ['\"' + str(v) + '\"' if ',' in str(v) else str(v) for v in row._asdict().values()]
+            file.write(',\t'.join(vals) + '\n')
+
+    def _write_header(self):
+        "Writes the column names as header to the file, separated by commas. Called automatically during init."
+        with open(self.path, 'w') as file:
+            file.write(','.join(self.Row._fields) + '\n')
+
+    def read_file(self, *args):
+        raise NotImplementedError('Use pandas.read_csv or csv directly.')
+
+    def read(self, *args):
+        raise NotImplementedError('Use pandas.read_csv or csv directly.')
+
+
 class Precomputed(list):
     """
     This class is a list of pre-computed sound stimuli which simplifies their generation and presentation. It is
@@ -1176,7 +1266,6 @@ def load_config(filename):
         conf.speeds
         # Out: [60, 120, 180]
     """
-    from collections import namedtuple
     with open(filename, 'r') as f:
         lines = f.readlines()
     if lines:
@@ -1186,5 +1275,5 @@ def load_config(filename):
             var, val = line.strip().split('=')
             var_names.append(var.strip())
             values.append(eval(val.strip()))
-        config_tuple = namedtuple('config', var_names)
+        config_tuple = collections.namedtuple('config', var_names)
         return config_tuple(*values)
